==> gist-angular-popovers-1.0.5.tgz: package/src/ts/gist-popovers/component/gist-popover-displayer/gist-popover-displayer.component.ts <==
import { PopoverEntry } from './../../classes/popover-entry.class';
import { Component, ViewContainerRef, Input, ViewChild, ElementRef, HostListener } from '@angular/core';

let windowOffset = 5;

@Component( {
    selector: 'gist-popover-displayer',
    templateUrl: './gist-popover-displayer.component.html',
    styleUrls: [ './gist-popover-displayer.component.css' ]
} )
export class GistPopoverDisplayerComponent {
    @Input()
    entry: PopoverEntry;

    @ViewChild( 'target', { read: ViewContainerRef } )
    target: ViewContainerRef;

    private viewTimer: number = undefined;
    @HostListener( 'mouseenter' )
    onMouseEnter() {
        this.entry.isActive = true;
        if ( this.viewTimer ) {
            window.clearTimeout( this.viewTimer );
            this.viewTimer = undefined;
        }
    }

    @HostListener( 'mouseleave' )
    onMouseLeave() {
        if ( this.viewTimer === undefined ) {
            this.viewTimer = window.setTimeout(() => {
                this.entry.isActive = false;
                this.viewTimer = undefined;
            }, this.entry.delay );
        }
    }

    private anchor: string;

    private anchorCoords: number[] = [ 0, 0 ];
    private offsetCoords: number[] = [ 0, 0 ];
    private windowOffsetCoords: number[] = [ 0, 0 ];

    private lastCoords: number[] = [ 0, 0 ];

    private ele: HTMLElement;

    constructor(
        private viewContainer: ViewContainerRef,
        private eleRef: ElementRef
    ) {
        this.ele = this.eleRef.nativeElement;
    }

    ngAfterViewInit() {
        let me = this;
        me.target.createEmbeddedView( me.entry.template );

        me.entry.anchorSide.subscribe( anc => {
            if ( me.anchor !== anc ) {
                me.anchor = anc;
                me.updateOffset();
                me.throttledUpdate();
            }
        } );
        me.entry.xPos.subscribe( x => {
            if ( me.anchorCoords[ 0 ] !== x ) {
                me.anchorCoords[ 0 ] = x;
                me.throttledUpdate();
            }
        } );
        me.entry.yPos.subscribe( y => {
            if ( me.anchorCoords[ 1 ] !== y ) {
                me.anchorCoords[ 1 ] = y;
                me.throttledUpdate();
            }
        } );
    }

    private updateOffset() {
        let me = this;
        let parent = me.ele.parentElement;
        if ( !parent ) {
            return;
        }

        if ( me.anchor === 'left' ) {
            me.offsetCoords[ 0 ] = me.ele.clientWidth * -1;
            me.offsetCoords[ 1 ] = me.ele.clientHeight * -0.5;
        }
        else if ( me.anchor === 'right' ) {
            me.offsetCoords[ 0 ] = 0;
            me.offsetCoords[ 1 ] = me.ele.clientHeight * -0.5;
        }
        else if ( me.anchor === 'top' ) {
            me.offsetCoords[ 0 ] = me.ele.clientWidth * -0.5;
            me.offsetCoords[ 1 ] = me.ele.clientHeight * -1;
        }
        else if ( me.anchor === 'bottom' ) {
            me.offsetCoords[ 0 ] = me.ele.clientWidth * -0.5;
            me.offsetCoords[ 1 ] = 0;
        }
    }

    private updateWindowOffset() {
        let me = this;
        let parent = me.ele.parentElement;
        if ( !parent ) {
            return;
        }

        let myBounds = me.ele.getBoundingClientRect();
        let parentBounds = parent.getBoundingClientRect();

        let overflowRight: number = parentBounds.right - myBounds.right;
        let overflowBottom: number = parentBounds.height - myBounds.bottom;

        // dont go past the left side
        if ( myBounds.left < windowOffset ) {
            if ( myBounds.left < 0 ) {
                me.windowOffsetCoords[ 0 ] = windowOffset - myBounds.left;
            } else {
                me.windowOffsetCoords[ 0 ] = windowOffset;
            }

        }
        // dont go past the right
        if ( overflowRight < windowOffset ) {
            me.windowOffsetCoords[ 0 ] = ( Math.abs( overflowRight ) * -2 ) - windowOffset;
        }

        // dont go past the top
        if ( myBounds.top < windowOffset ) {
            if ( myBounds.top < 0 ) {
                me.windowOffsetCoords[ 1 ] = windowOffset - myBounds.top;
            } else {
                me.windowOffsetCoords[ 1 ] = windowOffset;
            }
        }

        // dont go past the bottom
        if ( overflowBottom < windowOffset ) {
            me.windowOffsetCoords[ 1 ] = overflowBottom - windowOffset;
        }

        setTimeout(() => {
            me.updatePos.apply( me );
        }, 1 );
    }

    private throttle: number = undefined;
    private throttledUpdate() {
        let me = this;
        if ( me.throttle === undefined ) {
            me.throttle = window.setTimeout(() => {
                me.updatePos.apply( me );
                me.throttle = undefined;
            }, this.entry.delay );
        }
    }

    private updatePos() {
        let me = this;
        let coords = [
            Math.round( me.anchorCoords[ 0 ] + me.offsetCoords[ 0 ] + me.windowOffsetCoords[ 0 ] ),
            Math.round( me.anchorCoords[ 1 ] + me.offsetCoords[ 1 ] + me.windowOffsetCoords[ 1 ] )
        ];

        let parent = me.ele.parentElement;
        if ( parent ) {
            me.ele.style.maxWidth = ( parent.clientWidth - windowOffset * 2 ) + 'px';
            me.ele.style.maxHeight = ( parent.clientHeight - windowOffset * 2 ) + 'px';
        }

        // dont move it if its already there
        if ( coords[ 0 ] !== me.lastCoords[ 0 ] || coords[ 1 ] !== me.lastCoords[ 1 ] ) {
            me.ele.style.left = ( coords[ 0 ] ) + 'px';
            me.ele.style.top = ( coords[ 1 ] ) + 'px';

            me.lastCoords = coords;

            // ele width can change after rendering so check it again
            setTimeout(() => {
                me.ele.style.width = me.ele.clientWidth + 'px';
                me.ele.style.height = me.ele.clientHeight + 'px';
                // reset window offset
                me.updateWindowOffset.apply( me );
                me.ele.style.visibility = 'visible';
            }, 1 );
        }
    }

}

==> gist-angular-popovers-1.0.13.tgz: package/src/ts/gist-popovers/component/gist-popover-displayer/gist-popover-displayer.component.ts <==
import { PopoverEntry } from './../../classes/popover-entry.class';
import { Component, ViewContainerRef, Input, ViewChild, ElementRef, HostListener } from '@angular/core';

let windowOffset = 5;
let sizeChangeBuffer = 5;
@Component( {
    selector: 'gist-popover-displayer',
    templateUrl: './gist-popover-displayer.component.html',
    styleUrls: [ './gist-popover-displayer.component.css' ]
} )
export class GistPopoverDisplayerComponent {
    @Input()
    entry: PopoverEntry;

    @ViewChild( 'target', { read: ViewContainerRef } )
    target: ViewContainerRef;

    private viewTimer: number = undefined;
    @HostListener( 'mouseenter' )
    onMouseEnter() {
        this.entry.isActive = true;
        if ( this.viewTimer ) {
            window.clearTimeout( this.viewTimer );
            this.viewTimer = undefined;
        }
    }

    @HostListener( 'mouseleave' )
    onMouseLeave() {
        if ( this.viewTimer === undefined ) {
            this.viewTimer = window.setTimeout(() => {
                this.entry.isActive = false;
                this.viewTimer = undefined;
            }, this.entry.delay );
        }
    }

    private anchor: string;

    private anchorCoords: number[] = [ 0, 0 ];
    private offsetCoords: number[] = [ 0, 0 ];
    private windowOffsetCoords: number[] = [ 0, 0 ];

    private lastCoords: number[] = [ 0, 0 ];
    private lastOffsetCoords: number[] = [ 0, 0 ];

    private ele: HTMLElement;
    private checkInterval: number;

    constructor(
        private viewContainer: ViewContainerRef,
        private eleRef: ElementRef
    ) {
        this.ele = this.eleRef.nativeElement;
        this.ele.style.visibility = 'hidden';
    }

    ngAfterViewInit() {
        let me = this;
        me.target.createEmbeddedView( me.entry.template );

        me.entry.anchorSide.subscribe( anc => {
            if ( me.anchor !== anc ) {
                me.anchor = anc;
                me.throttledUpdate();
            }
        } );
        me.entry.xPos.subscribe( x => {
            if ( me.anchorCoords[ 0 ] !== x ) {
                me.anchorCoords[ 0 ] = x;
                me.throttledUpdate();
            }
        } );
        me.entry.yPos.subscribe( y => {
            if ( me.anchorCoords[ 1 ] !== y ) {
                me.anchorCoords[ 1 ] = y;
                me.throttledUpdate();
            }
        } );
    }

    ngOnDestroy() {
        window.clearTimeout( this.checkInterval );
    }

    private updateOffset() {
        let me = this;
        let parent = me.ele.parentElement;
        if ( !parent ) {
            return;
        }

        if ( me.anchor === 'left' ) {
            me.offsetCoords[ 0 ] = me.ele.clientWidth * -1;
            me.offsetCoords[ 1 ] = me.ele.clientHeight * -0.5;
        }
        else if ( me.anchor === 'right' ) {
            me.offsetCoords[ 0 ] = 0;
            me.offsetCoords[ 1 ] = me.ele.clientHeight * -0.5;
        }
        else if ( me.anchor === 'top' ) {
            me.offsetCoords[ 0 ] = me.ele.clientWidth * -0.5;
            me.offsetCoords[ 1 ] = me.ele.clientHeight * -1;
        }
        else if ( me.anchor === 'bottom' ) {
            me.offsetCoords[ 0 ] = me.ele.clientWidth * -0.5;
            me.offsetCoords[ 1 ] = 0;
        }

        me.offsetCoords[ 0 ] = +me.offsetCoords[ 0 ].toFixed( 2 );
        me.offsetCoords[ 1 ] = +me.offsetCoords[ 1 ].toFixed( 2 );
    }

    private updateWindowOffset() {
        let me = this;
        let parent = me.ele.parentElement;
        if ( !parent ) {
            return;
        }

        let myBounds = me.ele.getBoundingClientRect();
        let parentBounds = parent.getBoundingClientRect();

        let height = myBounds.height;
        let width = myBounds.width;

        let x1 = me.anchorCoords[ 0 ] + me.offsetCoords[ 0 ];
        let x2 = x1 + width;

        let y1 = me.anchorCoords[ 1 ] + me.offsetCoords[ 1 ];
        let y2 = y1 + height;

        if ( x1 < windowOffset ) {
            me.windowOffsetCoords[ 0 ] = windowOffset - x1;
        }
        if ( x2 > parentBounds.right - windowOffset ) {
            me.windowOffsetCoords[ 0 ] = parentBounds.right - windowOffset - x2;
        }

        if ( y1 < windowOffset ) {
            me.windowOffsetCoords[ 1 ] = windowOffset - y1;
        }
        if ( y2 > parentBounds.bottom - windowOffset ) {
            me.windowOffsetCoords[ 1 ] = parentBounds.bottom - windowOffset - y2;
        }

        me.windowOffsetCoords[ 0 ] = +me.windowOffsetCoords[ 0 ].toFixed( 2 );
        me.windowOffsetCoords[ 1 ] = +me.windowOffsetCoords[ 1 ].toFixed( 2 );
        if ( me.anchorCoords[ 1 ] + me.offsetCoords[ 1 ] + me.windowOffsetCoords[ 1 ] < windowOffset ) {
            console.error( me.anchorCoords[ 1 ] + ' : ' + me.offsetCoords[ 1 ] + ' : ' + me.windowOffsetCoords[ 1 ] );
        }
    }

    private throttle: number = undefined;
    private throttledUpdate() {
        let me = this;
        if ( me.throttle === undefined ) {
            me.updateOffset();
            me.throttle = window.setTimeout(() => {
                me.updatePos();
                me.throttle = undefined;
            }, this.entry.delay );
        }
    }

    private updatePos() {
        let me = this;
        let coords = [
            Math.round( me.anchorCoords[ 0 ] + me.offsetCoords[ 0 ] + me.windowOffsetCoords[ 0 ] ),
            Math.round( me.anchorCoords[ 1 ] + me.offsetCoords[ 1 ] + me.windowOffsetCoords[ 1 ] )
        ];

        let parent = me.ele.parentElement;
        if ( parent ) {
            me.ele.style.maxWidth = ( parent.clientWidth - windowOffset * 2 ) + 'px';
            me.ele.style.maxHeight = ( parent.clientHeight - windowOffset * 2 ) + 'px';
        }
        me.isDrawing = false;

        // dont move it if its already there
        if ( coords[ 0 ] !== me.lastCoords[ 0 ] || coords[ 1 ] !== me.lastCoords[ 1 ] ) {
            me.ele.style.left = ( coords[ 0 ] ) + 'px';
            me.ele.style.top = ( coords[ 1 ] ) + 'px';

            me.lastCoords = coords;
            me.drawTilNoChange();
            // ele width can change after rendering so check it again

        } else {
            this.ele.style.visibility = 'visible';

            this.checkInterval = window.setTimeout(() => { me.drawTilNoChange.apply( me ); }, 1000 / 30 );
        }
    }
    private renderThen( cb: ( v: any ) => any ): void {
        let me = this;
        setTimeout(() => {
            // reset window offset
            cb.apply( me );
        }, 100 );
    }

    private isDrawing: boolean = false;
    private drawTilNoChange() {
        let me = this;
        if ( me.isDrawing ) {
            return;
        }
        me.isDrawing = true;
        me.renderThen(() => {
            me.updateOffset();
            me.ele.style.width = me.ele.clientWidth + 'px';
            me.renderThen(() => {
                me.updateWindowOffset();
                me.updatePos();
            } );
        } );
    }

}
